package com.persagy.dptool;

import com.persagy.dptool.dto.*;

import java.util.HashSet;
import java.util.Set;

public class DictionaryUtil {

    /**
     * 解析专业-系统-设备-部件json配置文件转为的map数据
     * @param jsonString
     * @param sbStr
     * @return
     */
    public static FacilityDTO readFacility_new(String jsonString, StringBuilder sbStr) {
        final String fileName = "Facility_new.json文件";
        FacilityDTO result = null;
        String errorString = "";

        try {
           result = CommonUtil.jsonStrToObj(jsonString, FacilityDTO.class);
        }catch (Exception e) {}

        if(null == result || result.getAll() == null || result.getAll().size() == 0) {
            errorString = "数据结构错误！请参考标准数据字典改文件结构。";
        }else {
            Set<String> majorCodeSet = new HashSet<>();
            String majorCode = null;
            for(MajorDTO major : result.getAll()) {
                if(major == null) {
                    errorString = "数据结构错误！请参考标准数据字典改文件结构。";
                    break;
                }

                String checkResult = major.check();
                if(null != checkResult) {
                    errorString = "有误！"+checkResult;
                    break;
                }

                majorCode = major.getCode();
                if(majorCodeSet.contains(majorCode)) {
                    errorString = "有误！有重复的专业编码" + majorCode;
                    break;
                }else {
                    majorCodeSet.add(majorCode);
                }

                if(major.getContent() != null && major.getContent().size() > 0) {
                    Set<String> systemCodeSet = new HashSet<>();
                    String systemCode = null;
                    for(SystemDTO system : major.getContent()) {
                        if(system == null) {
                            errorString = "有误！"+majorCode+"专业下有不合法的系统类型配置信息！";
                            break;
                        }

                        checkResult = system.check();
                        if(null != checkResult) {
                            errorString = "有误！"+majorCode+"专业下配置信息错误："+checkResult;
                            break;
                        }

                        systemCode = system.getCode();
                        if(systemCodeSet.contains(systemCode)) {
                            errorString = "有误！"+majorCode + "专业下有重复的系统编码" + systemCode;
                            break;
                        }else {
                            systemCodeSet.add(systemCode);
                        }

                        if(system.getContent() != null && system.getContent().size() > 0) {
                            Set<String> eqCodeSet = new HashSet<>();
                            String eqCode = null;
                            for(EqDTO eqdto : system.getContent()) {
                                if(eqdto == null) {
                                    errorString = "有误！"+majorCode+"专业"+systemCode+"系统下有不合法的设备类型配置信息！";
                                    break;
                                }

                                checkResult = eqdto.check();
                                if(null != checkResult) {
                                    errorString = "有误！"+majorCode+"专业"+systemCode+"系统下有不合法的设备类型配置信息："+checkResult;
                                    break;
                                }

                                eqCode = eqdto.getCode();
                                if(eqCodeSet.contains(eqCode)) {
                                    errorString = "有误！"+majorCode+"专业"+systemCode+"系统下有重复的设备编码" + eqCode;
                                    break;
                                }else {
                                    eqCodeSet.add(eqCode);
                                }

                                if(eqdto.getComponents() != null && eqdto.getComponents().size() > 0) {
                                    Set<String> compCodeSet = new HashSet<>();
                                    String compCode = null;
                                    for(ComponentDTO comp : eqdto.getComponents()) {
                                        if(comp == null) {
                                            errorString = "有误！"+majorCode+"专业"+systemCode+"系统"+eqCode+"设备下有不合法的部件类型配置信息！";
                                            break;
                                        }

                                        checkResult = comp.check();
                                        if(null != checkResult) {
                                            errorString = "有误！"+majorCode+"专业"+systemCode+"系统"+eqCode+"设备下有不合法的部件类型配置信息："+checkResult;
                                            break;
                                        }

                                        compCode = comp.getCode();
                                        if(compCodeSet.contains(compCode)) {
                                            errorString = "有误！"+majorCode+"专业"+systemCode+"系统"+eqCode+"设备下有重复的部件编码" + compCode;
                                            break;
                                        }else {
                                            compCodeSet.add(compCode);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        if(errorString.length() > 0) {
            sbStr.append(CommonUtil.getLineString(fileName+errorString, 0));
        }else {
            result.normal = true;
        }

        return result;
    }
}
