package com.persagy.dptool;

import javafx.concurrent.Task;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.stage.DirectoryChooser;

import java.io.File;
import java.net.URL;
import java.util.ResourceBundle;

public class PrimaryController implements Initializable {
    @FXML
    private BorderPane paneRoot;
    @FXML
    private TabPane paneTab;
    /** 配置文件目录 */
    @FXML
    public TextField txfDir;
    @FXML
    public Button btnSelectDir;
    @FXML
    public Button btnCheck;
    /** 校验结果文本域 */
    @FXML
    public TextArea txaContent;
    @FXML
    public ProgressIndicator piState;
    /** 底部状态栏 */
    @FXML
    public Label lblState;


    @Override
    public void initialize(URL location, ResourceBundle resources) {
        piState.setProgress(-1);
        piState.setVisible(false);
        lblState.setText("");
    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectDir(ActionEvent e) {
        DirectoryChooser dirChooser =new DirectoryChooser();
        dirChooser.setTitle("选择配置文件目录");

        File folder = dirChooser.showDialog(paneRoot.getScene().getWindow());

        if(null != folder) {
            txfDir.setText(folder.getAbsolutePath());
        }
    }

    /**
     * 检查配置文件
     * @param e
     */
    public void checkConfig(ActionEvent e) {
        File configFile = getFile(txfDir.getText(), "config.properties");
        File propertyFile = getFile(txfDir.getText(), "property");

        if(null == configFile && propertyFile == null) {
            txaContent.setText("未找到需要校验的config.properties文件和property数据字典文件夹，请确保所选目录是否正确。");
            return;
        }

        Task<Boolean> task = TaskFactory.checkConfig(this, configFile, propertyFile);
        new Thread(task).start();
    }

    /**
     * 根据文件路径获取文件实例
     * @param dirPath
     * @return
     */
    private File getFile(String dirPath, String fileName) {
        File file = null;

        if(dirPath != null) {
            dirPath = dirPath.trim() + File.separator + fileName;
            file = new File(dirPath);
            if(!file.exists()) {
                file = null;
            }
        }
        return file;
    }

    /**
     * 设置tab页签中的控件是否可用
     * @param disable true-可用
     * @param typeIndex 从1开始
     */
    public void setDisable(boolean disable, int typeIndex) {
        if(1 == typeIndex) {
            paneTab.setDisable(disable);
        }

    }
}
