package com.persagy.dptool;

import com.persagy.dptool.dto.EqDTO;
import com.persagy.dptool.dto.FacilityDTO;
import com.persagy.dptool.dto.MajorDTO;
import com.persagy.dptool.dto.SystemDTO;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import javafx.application.Platform;
import javafx.concurrent.Task;

import javax.jms.TopicSession;
import java.io.File;
import java.util.*;

public class TaskFactory {
    public static String[] prefixStr = {"- Error", "- Warn"};

    public static Task<Boolean> checkConfig(PrimaryController controller, File configFile, File propertyFile) {
        return new Task<Boolean>() {
            @Override
            protected Boolean call() throws Exception {
                Platform.runLater(()->{
                    controller.txaContent.setText("");
                    controller.setDisable(true, 1);
                    controller.piState.setVisible(true);
                    controller.lblState.setText("准备校验...");
                });

                if(configFile != null && configFile.isFile()) {
                    Platform.runLater(()->{
                        controller.lblState.setText("校验config.properties文件...");
                    });
                    Thread.sleep(500);

                    Map<String, String> configMap = CommonUtil.property2Map(configFile);
                    String propertyFileCheckResult = checkConfigFile(controller, configMap);

                    Platform.runLater(()->{
                        controller.txaContent.appendText(propertyFileCheckResult);
                    });
                }else {
                    Platform.runLater(()->{
                        controller.lblState.setText("未找到config.properties文件，准备校验数据字典文件...");
                    });
                    Thread.sleep(500);
                }

                if(propertyFile != null && propertyFile.isDirectory()) {
                    Platform.runLater(()->{
                        controller.lblState.setText("校验数据字典配置文件...");
                    });
                    Thread.sleep(500);

                    String dictionaryCheckResult = checkDictionary(propertyFile);

                    Platform.runLater(()->{
                        controller.txaContent.appendText(dictionaryCheckResult);
                    });
                }else {
                    Platform.runLater(()->{
                        controller.lblState.setText("数据字典配置文件目录:/property 未找到。");
                    });
                }

                Platform.runLater(()->{
                    controller.setDisable(false, 1);
                    controller.piState.setVisible(false);
                    controller.lblState.setText("");
                });
                return true;
            }
        };
    }

    /**
     * 校验数据字典配置文件
     * @param propertyFile property数据字典配置文件目录
     * @return
     */
    private static String checkDictionary(File propertyFile) {
        StringBuilder sbStr = new StringBuilder("【数据字典配置信息校验结果】\n");

        File[] folders = propertyFile.listFiles();

        if(null != folders && folders.length > 0) {
            Set<String> noFolderSet = new HashSet<>(ConstantData.dicFolderBase);
            Map<String, File> hasFolderMap = getName2FileMap(folders, ConstantData.dicFolderBase);

            if(hasFolderMap.keySet().size() != ConstantData.dicFolderBase.size()) {
                noFolderSet.removeAll(hasFolderMap.keySet());
                sbStr.append(CommonUtil.getLineString("目录property下缺少必要子目录" + noFolderSet, 0));
            }else {
                FacilityDTO facilityDTO = checkDictionaryOnDictionary(sbStr, hasFolderMap.get(ConstantData.dicFolderDictionary));
                if(facilityDTO != null && facilityDTO.normal) {
                    checkDictionaryOnInfoCodeByFacilityDTO(hasFolderMap.get(ConstantData.dicFolderInfoCode), sbStr, facilityDTO);
                }else {
                    checkDictionaryOnInfoCode(hasFolderMap.get(ConstantData.dicFolderInfoCode), sbStr);
                }
            }
        }else {
            sbStr.append(CommonUtil.getLineString("目录property下缺少必要子目录" + ConstantData.dicFolderBase, 0));
        }

        String checkResult = sbStr.toString();
        if(checkResult.contains("Error") || checkResult.contains("Warn")) {
            checkResult = checkResult + "\n";
        }else {
            checkResult = checkResult + "未发现异常配置。\n\n";
        }

        return checkResult;
    }

    /**
     * 先校验InfoCode目录下的所有json文件是否为合法的json数据结构，再按照FacilityDTO实例包含的专业-系统-设备-备件组织结构信息校验相关文件。发现错误则不继续校验。
     * @param file
     * @param sbStr
     * @param facilityDTO
     */
    private static void checkDictionaryOnInfoCodeByFacilityDTO(File file, StringBuilder sbStr, FacilityDTO facilityDTO) {
        if(file == null || !file.isDirectory()) {
            return;
        }
        StringBuilder mySbStr = new StringBuilder("");
        checkDictionaryOnInfoCode(file, mySbStr);
        if(mySbStr.length() > 0) {
            sbStr.append(mySbStr);
        }else {
            List<MajorDTO> majorList = facilityDTO.getAll();
            Map<String, MajorDTO> majorCode2DTO = new HashMap<>();
            majorList.forEach((item)->majorCode2DTO.put(item.getCode(), item));

            Map<String, File> majorCode2File = new HashMap<>();
            File[] majorFolders = file.listFiles();
            for(File majorFolder : majorFolders) {
                if(majorFolder.isDirectory() && majorCode2DTO.keySet().contains(majorFolder.getName())) {
                    majorCode2File.put(majorFolder.getName(), majorFolder);
                }
            }

            if(majorList.size() != majorCode2File.size()) {
                Set<String> allMajorCodeSet = new HashSet<>(majorCode2DTO.keySet());
                allMajorCodeSet.removeAll(majorCode2File.keySet());
                sbStr.append(CommonUtil.getLineString("/property/InfoCode目录下缺少以下专业编码的配置目录："+allMajorCodeSet, 0));
            }else {
                for(MajorDTO majorDTO : majorList) {
                    List<SystemDTO> systemList = majorDTO.getContent();
                    if(null == systemList || systemList.size() == 0) {
                        continue;
                    }

                    Map<String, SystemDTO> systemCode2DTO = new HashMap<>();
                    systemList.forEach((item)->systemCode2DTO.put(item.getCode(), item));

                    Map<String, File> systemCode2File = new HashMap<>();
                    File[] systemFolders = majorCode2File.get(majorDTO.getCode()).listFiles();
                    for(File systemFolder : systemFolders) {
                        if(systemFolder.isDirectory() && systemCode2DTO.keySet().contains(systemFolder.getName())) {
                            systemCode2File.put(systemFolder.getName(), systemFolder);
                        }
                    }

                    if(systemList.size() != systemCode2File.size()) {
                        Set<String> allSystemCodeSet = new HashSet<>(systemCode2DTO.keySet());
                        allSystemCodeSet.removeAll(systemCode2File.keySet());
                        sbStr.append(CommonUtil.getLineString("/property/InfoCode/"+majorDTO.getCode()+"目录下缺少以下系统编码的配置目录："+allSystemCodeSet, 0));
                        break;
                    }else {
                        for(SystemDTO systemDTO : systemList) {
                            if(null == systemDTO.getContent() || systemDTO.getContent().size() == 0) {
                                continue;
                            }else {
                                Set<String> jsonFileNameSet = new HashSet<>();
                                for(EqDTO eqDTO : systemDTO.getContent()) {
                                    jsonFileNameSet.add(eqDTO.getCode()+".json");
                                    if(eqDTO.getComponents() != null && eqDTO.getComponents().size() > 0) {
                                        eqDTO.getComponents().forEach((item)->jsonFileNameSet.add(item.getCode()+".json"));
                                    }
                                }

                                File[] jsonFiles = systemCode2File.get(systemDTO.getCode()).listFiles();
                                Set<String> realJsonFileSet = new HashSet<>();
                                for(File jsonFile : jsonFiles) {
                                    if(jsonFileNameSet.contains(jsonFile.getName())) {
                                        realJsonFileSet.add(jsonFile.getName());
                                    }
                                }

                                if(realJsonFileSet.size() != jsonFileNameSet.size()) {
                                    jsonFileNameSet.removeAll(realJsonFileSet);
                                    sbStr.append(CommonUtil.getLineString("/property/InfoCode/"+majorDTO.getCode()+"/"+systemDTO.getCode()+"目录下缺少以下json配置文件："+jsonFileNameSet, 0));
                                    return;
                                }
                            }
                        }
                    }
                }
            }

        }
    }

    /**
     * 校验数据字典InfoCode配置目录下的json文件
     * @param file 配置文件目录实例
     * @param sbStr
     */
    private static void checkDictionaryOnInfoCode(File file, StringBuilder sbStr) {
        if(file == null) {
            return;
        }

        if(file.isFile() && file.getName().endsWith(".json")) {
            String filePath = file.getAbsolutePath();
            filePath = filePath.substring(filePath.indexOf("property"));
            try {
                CommonUtil.jsonStrToMap(CommonUtil.readFileToString(file));
            }catch (Exception e) {
                sbStr.append(CommonUtil.getLineString(filePath+"文件内容不是合法的json数据结构", 0));
                return;
            }
        }else if(file.isDirectory()) {
            for(File sonFile : file.listFiles()) {
                checkDictionaryOnInfoCode(sonFile, sbStr);
            }
        }
    }


    /**
     * 获取standSet集合中含有的文件名称与文件实例关系
     * @param files
     * @param standSet
     * @return
     */
    private static Map<String, File> getName2FileMap(File[] files, Set<String> standSet) {
        Map<String, File> hasFolderMap = new HashMap<>();
        for(File folder : files) {
            String fileName = null;
            if(folder != null) {
                fileName = folder.getName();

                if(!standSet.contains(fileName)) {
                    continue;
                }

                hasFolderMap.put(fileName, folder);
            }

            if(hasFolderMap.keySet().size() == standSet.size()) {
                break;
            }
        }

        return hasFolderMap;
    }

    /**
     * 校验数据字典Dictionary目录文件
     * @param sbStr
     * @param folder
     * @return 专业系统设备部件组织结构
     */
    private static FacilityDTO checkDictionaryOnDictionary(StringBuilder sbStr, File folder) {
        final String thisDir = "/property/Dictionary";
        final String jsonError = "文件内容不是合法的json数据结构";

        FacilityDTO result = null;

        File[] files = folder.listFiles();

        if(null != files && files.length > 0) {
            Set<String> noFileSet = new HashSet<>(ConstantData.jsonFilesOfDictionary);
            Map<String, File> hasFileMap = getName2FileMap(files, ConstantData.jsonFilesOfDictionary);

            if(hasFileMap.keySet().size() != ConstantData.jsonFilesOfDictionary.size()) {
                noFileSet.removeAll(hasFileMap.keySet());
                String noFilesInfo = noFileSet.toString();
                String infoStr = CommonUtil.getLineString("目录"+thisDir+"下缺少必要json配置文件" + noFilesInfo, 0);
                infoStr = infoStr.replace("[", "").replace("]", "。");
                sbStr.append(infoStr);
            }else {
                for(String fileName : hasFileMap.keySet()) {
                    Map<String, Object> dataMap = null;
                    String jsonStr = null;
                    try {
                        jsonStr = CommonUtil.readFileToString(hasFileMap.get(fileName));
                        dataMap = CommonUtil.jsonStrToMap(jsonStr);
                    }catch (Exception e) {
                        dataMap = null;
                    }

                    if(dataMap == null) {
                        sbStr.append(CommonUtil.getLineString(thisDir+"/"+fileName+jsonError, 0));
                    }else {
                        // 具体解析json配置文件

                        if("Facility_new.json".equals(fileName)) {
                            // 解析专业-系统-设备-部件组织结构
                            result = DictionaryUtil.readFacility_new(jsonStr, sbStr);
                        }
                    }
                }
            }
        }else {
            sbStr.append(CommonUtil.getLineString("目录"+thisDir+"下配置文件缺失", 0));
        }

        return result;
    }


    /**
     * 校验数据平台properties配置文件
     * @param controller
     * @param configMap
     */
    private static String checkConfigFile(PrimaryController controller, Map<String, String> configMap) {
        StringBuilder sbStr = new StringBuilder("【config.properties配置信息校验结果】\n");

        String dictSource = configMap.get("dict.source");
        if(dictSource == null) {
            sbStr.append(CommonUtil.getLineString("配置项dict.source缺失；", 0));
        }else if(!"local".equals(dictSource)) {
            sbStr.append(CommonUtil.getLineString("配置项dict.source未使用本地数据字典配置；",1));
        }

        String jmsActive = configMap.get("jms.active");
        if("false".equals(jmsActive)) {
            sbStr.append(CommonUtil.getLineString("消息中间件配置未打开；", 0));
        }else {
            String jmsChoice = configMap.get("jms.choice");
            if("rabbit".equals(jmsChoice)) {
                // 使用的是rabbitMq中间件
                sbStr.append(checkRabbitMq(configMap));
            }else {
                // 使用是activeMq中间件
                sbStr.append(checkActiveMQ(configMap));
            }
        }

        String imgServiceUrl = configMap.get("image.service.url");
        if(imgServiceUrl == null) {
            sbStr.append(CommonUtil.getLineString("配置项image.service.url缺失；", 0));
        }else {
            String imgSystemId = configMap.get("image.service.systemId");
            String imgSecret= configMap.get("image.service.secret");
            if(null != imgSecret && imgSystemId != null) {
                if(!imgSecret.equals(ConstantData.imgKeyMap.get(imgSystemId))) {
                    sbStr.append(CommonUtil.getLineString("文件服务systemId与secret不匹配；", 0));
                }else {
                    if(!fileGetTest(imgServiceUrl, imgSystemId)) {
                        sbStr.append(CommonUtil.getLineString("文件服务访问不通！image-service Address="+imgServiceUrl+"请确保生产环境数据平台可访问文件服务；", 0));
                    }
                }
            }else {
                if(null == imgSystemId) {
                    sbStr.append(CommonUtil.getLineString("配置项image.service.systemId缺失；", 0));
                }
                if(null == imgSecret) {
                    sbStr.append(CommonUtil.getLineString("配置项image.service.secret缺失；", 0));
                }
            }
        }

        String checkResult = sbStr.toString();
        if(checkResult.contains("Error") || checkResult.contains("Warn")) {
            checkResult = checkResult + "\n";
        }else {
            checkResult = checkResult + "未发现异常。\n\n";
        }

        return checkResult;
    }

    private static String checkActiveMQ(Map<String, String> configMap) {
        String result = "";

        String brokerurl = configMap.get("jms.brokerurl");
        String userName = configMap.get("jms.userName");
        String password = configMap.get("jms.password");
        String topic = configMap.get("jms.topic");

        if(brokerurl == null || userName == null || password == null || topic == null) {
            result += CommonUtil.getLineString("ActiveMq必有配置项值不能为空！请到FTP下载标准数据平台程序，参考其配置文件内容；", 0);
            return result;
        }

        if(!"dataPlatform.broadcast".equals(topic)) {
            result += CommonUtil.getLineString("ActiveMq配置项jms.topic值不是默认值dataPlatform.broadcast；", 1);
        }

        try {
            javax.jms.ConnectionFactory factory = new org.apache.activemq.ActiveMQConnectionFactory(userName, password, brokerurl);
            javax.jms.Connection connection = factory.createConnection();
            connection.start();
            javax.jms.Session session = connection.createSession(false, TopicSession.AUTO_ACKNOWLEDGE);
            javax.jms.Topic topicObj = session.createTopic(topic);
            javax.jms.MessageProducer messageProducer = session.createProducer(topicObj);
            messageProducer.setTimeToLive(1000 * 60 * 60 * 2);
        }catch (Exception e) {
            e.printStackTrace();
            result += CommonUtil.getLineString("ActiveMq消息中间件服务访问不通！errorMsg=" + e.getMessage() + " 请确保生产环境消息中间件服务能够正常访问。", 0);
        }

        return result;
    }

    /**
     * 检查rabbitMQ服务
     * @param configMap
     * @return
     */
    private static String checkRabbitMq(Map<String, String> configMap) {
        String result = "";
        String host = configMap.get("rabbit.host");
        String port = configMap.get("rabbit.port");
        String virtualhost = configMap.get("rabbit.virtualhost");
        String userName = configMap.get("rabbit.userName");
        String password = configMap.get("rabbit.password");
        String topic = configMap.get("rabbit.topic");
        String routingKey = configMap.get("rabbit.routingKey");

        if(host == null || port == null || virtualhost == null || userName == null || password == null || topic == null || routingKey == null) {
            result += CommonUtil.getLineString("RabbitMq必有配置项值不能为空！请到FTP下载标准数据平台程序，参考其配置文件内容；", 0);
            return result;
        }

        if(!"dataPlatform.broadcast".equals(topic)) {
            result += CommonUtil.getLineString("RabbitMq配置项rabbit.topic值不是默认值dataPlatform.broadcast；", 1);
        }

        if(!"dataPlatform".equals(routingKey)) {
            result += CommonUtil.getLineString("RabbitMq配置项rabbit.routingKey值不是默认值dataPlatform；", 1);
        }

        try {
            ConnectionFactory factory = new ConnectionFactory();
            factory.setUsername(userName);
            factory.setPassword(password);
            factory.setHost(host);
            factory.setPort(Integer.parseInt(port));
            factory.setVirtualHost(virtualhost);

            Connection connection = factory.newConnection();
            Channel channel = connection.createChannel();
            channel.exchangeDeclare(topic, "topic", true);

//            channel.basicPublish(topic, routingKey, null, "data-platform-tool-testMsg".getBytes(Charset.forName("UTF-8")));

        }catch (Exception e) {
            e.printStackTrace();
            result += CommonUtil.getLineString("RabbitMq消息中间件服务访问不通！errorMsg=" + e.getMessage() + " 请确保生产环境消息中间件服务能够正常访问。", 0);
        }

        return result;
    }

    /**
     * 测试文件服务file_get接口
     * @param base
     * @param sysId
     * @return
     */
    private static boolean fileGetTest(String base, String sysId) {
        boolean result = false;
        String url = base + File.separator + "common/file_get?key=dPfToOltEsTkey&systemId=" + sysId;
        try {
            byte[] byteData = CommonUtil.httpGetFile(url);
            if(byteData != null && ifNotExists(byteData)) {
                result = true;
            }
        }catch (Exception e){}

        return result;
    }

    /**
     * 判断byte数组是否包含“not existed”字符串
     * @param byteData
     * @return true-包含；false-不包含
     */
    private static boolean ifNotExists(byte[] byteData) {
        String s = new String(byteData);
        return s.contains("not existed");
    }
}
